"""
Written by Matteo Dunnhofer - 2020

Class that defines our tracker with mask generation
"""
#import sys
#sys.path.insert(0, '..')
import path_config as pc
import numpy as np
import torch
from track.Tracker import TRASFUST
from demotrackers.AlphaRefine.arena.VOT2020.utils import bbox_clip


class TrackerMask(object):

    def __init__(self, tracker, ckpt_path, teacher_tracking=['SiamRPN'], teacher_training='POOL', lstm_update=False, gpu_id=0, args=None):
        super(TrackerMask, self).__init__()

        # tracker setup
        self.tracker = TRASFUST(ckpt_path, teacher_tracking, lstm_update=lstm_update, gpu_id=gpu_id, teacher_training=teacher_training)

        # AlphaRefine setup
        from demotrackers.AlphaRefine.pytrackingaf.refine_modules.refine_module_vot20 import RefineModule
        self.af = RefineModule(pc.CKPT_PATH + 'AlphaRefine.pth.tar')

    def init(self, image, box, **kwargs):
        self.tracker.init(image, box, **kwargs)

        # AlphaRefine
        img = np.array(image)
        self.H, self.W, _ = img.shape
        self.af.initialize(img, np.array(box))

    def update(self, image):
        tracker_pred = self.tracker.update(image)

        # AlphaRefine
        img = np.array(image)
        x1, y1, w, h = tracker_pred
        # add boundary and min size limit
        x1, y1, x2, y2 = bbox_clip(x1, y1, x1 + w, y1 + h, (self.H, self.W))
        w = x2 - x1
        h = y2 - y1

        bbox_new = [x1,y1,w,h]

        pred_mask = self.af.refine(img, np.array(bbox_new))
        mask = (pred_mask > 0.65).astype(np.uint8)

        return tracker_pred, mask, None
